//
// Frequency Monitor
//
// Copyright(c) 2002 Tatsuyuki Satoh , All rights reserved
//
// Important !
//
// 1. License
//
// This program is freeware for non-commercial use. 
// The user of this source must assume all legal 
// responsibility in the use of this program. 
// This is not GPL.
//
// 2. Guarantee 
//
// An author does no guarantee about this program.
// You can use this under your own risk. 
// 
// 3. A rule to distribute.
//   Do either 1) or 2).
//   1)Include a perfect original source.
//   2)Include all these headers , and add document of 
//     modification points
//
// Histry
// 2002.11.18 Project Start
// 2002.11.22 1st releaes
//

module freq_mon(
// OSC
	base_clk,
	update,
// FREQ. counter , BCD x6
	target_clk,
	target_freq
);
/* configration */
parameter DECIMEL    = 1;        // hexa / decimel mode select
parameter UPDATE_HZ  = 2;        // Display cycle
parameter BCLK_Hz    = 18432000; // base_clk frequency
parameter BCLK_BITS  = 25;       // base_clk count bits
parameter COUNT_Hz   = 100;      // count unit
parameter COUNT_BITS = 21;       // count bits

/* module port */
input  base_clk;
output update;
input  target_clk;
output [23:0] target_freq;

/* internal parameters */
parameter DISP_CYCLE = (BCLK_Hz/UPDATE_HZ);
parameter STEP_CYCLE = (COUNT_Hz/UPDATE_HZ);

reg [BCLK_BITS-1:0] cycle_cnt;

reg [23:0] latch_num;
reg [3:0] num6,num5,num4,num3,num2,num1;

reg clk_trg;
reg clk_sense , clk_trg1 , clk_trg2;
reg latch_cycle;

always @(posedge base_clk)
begin
  if( cycle_cnt == 0 )
  begin
    latch_cycle <= 1;
    cycle_cnt   <= DISP_CYCLE; /* +1 for latch cycle */

    /* update freq. number */
    latch_num <= {num6,num5,num4,num3,num2,num1};
    {num6,num5,num4,num3,num2,num1} <= 0;

	/* reset sense level */
    clk_sense <= 0;
  end else begin
    latch_cycle <= 0;
    cycle_cnt   <= cycle_cnt - 1;

    if( (clk_sense^clk_trg1) & (clk_sense^clk_trg2) )
    begin
      clk_sense <= ~clk_sense;
      // countup
if(DECIMEL)
begin
      // DEC mode
      if(num1!=9) num1 <= num1 + 1;
      else begin
        num1 <= 0;
        if(num2!=9) num2 <= num2 + 1;
        else begin
          num2 <= 0;
          if(num3!=9) num3 <= num3 + 1;
          else begin
            num3 <= 0;
            if(num4!=9) num4 <= num4 + 1;
            else begin
              num4 <= 0;
              if(num5!=9) num5 <= num5 + 1;
              else begin
                num5 <= 0;
                num6 <= num6 + 1;
              end
            end
          end
        end
      end
end else begin
      // HEX mode
      {num6,num5,num4,num3,num2,num1} <=
      {num6,num5,num4,num3,num2,num1} +1;
end
     end
  end
  clk_trg1 <= clk_trg;
  clk_trg2 <= clk_trg1;
end

/* count target CLK */
reg [COUNT_BITS-1:0] freq_low;

always @(posedge target_clk or posedge latch_cycle)
begin
  if(latch_cycle)
  begin
    freq_low <= STEP_CYCLE-1;
    clk_trg  <= 0;
  end else begin
    if(freq_low == 0)
    begin
      freq_low <= STEP_CYCLE-1;
      clk_trg  <= ~clk_trg;
    end else begin
      freq_low <= freq_low - 1;
    end
  end
end

assign update = latch_cycle;
assign target_freq = latch_num;

endmodule

