//		    
// XST005P : VCO tester
//
// Copyright(c) 2002 Tatsuyuki Satoh , All rights reserved
//
// Important !
//
// 1. License
//
// This program is freeware for non-commercial use. 
// The user of this source must assume all legal 
// responsibility in the use of this program. 
// This is not GPL.
//
// 2. Guarantee 
//
// An author does no guarantee about this program.
// You can use this under your own risk. 
// 
// 3. A rule to distribute.
//   Do either 1) or 2).
//   1)Include a perfect original source.
//   2)Include all these headers , and add document of 
//     modification points
//
// Histry
// 2002.11.15 Project Start
// 2002.11.22 1st releaes
//

module top(
  PSW,LED,XSG,SA,BUZZ,PCLK,  // XSP005 reference board
  XIN,XOUT,                  // X'TAL OSC
  VCO_GTL,VCO_TTL,VCO_INV,   // VCO
  PLL_OUT                    // PLL
);

/////////////////////////////////////////////////////////////////////////////
// I/O assign
/////////////////////////////////////////////////////////////////////////////

// XSP005
input [3:0] PSW;     // push switch
inout [7:0] LED;     // static LED x8
inout [7:0] XSG;     // 7SEG-LEDx4 segment
inout [3:0] SA;      // 7SEG-LEDx4 commons
output BUZZ;         // buzzer
input PCLK;          // 14.138MHz

//X'TAL
input  XIN;
output XOUT;
//VCO
input VCO_GTL,VCO_TTL;
output VCO_INV;
//PLL
output PLL_OUT;

/////////////////////////////////////////////////////////////////////////////
// 7segment dynamic LED driver
/////////////////////////////////////////////////////////////////////////////
wire [3:0] dot;
wire [1:0] led_com;
wire led_en;
wire [3:0] led_cas;
wire [15:0] led_num;

hex4_led seg_led(.HEX4(led_num),.DOT(dot),.CNT(led_com),.EN(led_en),.SEG(XSG),.COM(led_cas) );

assign SA[3] = led_cas[0] ? 1'bz : 1'b0;
assign SA[2] = led_cas[1] ? 1'bz : 1'b0;
assign SA[1] = led_cas[2] ? 1'bz : 1'b0;
assign SA[0] = led_cas[3] ? 1'bz : 1'b0;

/////////////////////////////////////////////////////////////////////////////
// ticker
////////////////////////////////////////////////////////////////////////////

// tick counter (PCLK = 18.432MHz)
// ticker[16] = @140Hz
// ticker[24] = @1Hz
reg  [16:0] ticker;

always @(posedge PCLK)
begin
  ticker <= ticker + 1;
end

/////////////////////////////////////////////////////////////////////////////
// X'tal OSC
////////////////////////////////////////////////////////////////////////////
wire x_inv;
IBUF     x_in(.I(XIN),.O(x_inv));
OBUF_F_2 x_out(.O(XOUT),.I(~x_inv));

/////////////////////////////////////////////////////////////////////////////
// VCO
/////////////////////////////////////////////////////////////////////////////

wire vco_clk;      // VCO clock
wire in_t , in_g;  // TTL/GTL input level
wire in_p;         
wire in_n = ~in_p;

// SR latch , preset is in_t=H , clear is in_g=L
FDCP in_fb(.Q(in_p),.C(1'b0),.CLR(~in_g),.D(1'b0),.PRE(in_t) );

// LoopBack OSC.
IBUF_GTLP VCO_G(.I(VCO_GTL),.O(in_g)); // in (HIGH)
IBUF      VCO_T(.I(VCO_TTL),.O(in_t)); // in (LOW)
OBUF_F_4  VCO_O(.O(VCO_INV),.I(in_n)); // out (invert)

// GrobalClockDrive
BUFG VCLK(.I(in_p),.O(vco_clk));

/////////////////////////////////////////////////////////////////////////////
// PLL
/////////////////////////////////////////////////////////////////////////////

// 25MHz
parameter LOOPA_25 = 2304-1;
parameter LOOPB_25 = 3125-1;

// 33MHz
parameter LOOPA_33 = 2304-1;
parameter LOOPB_33 = 4125-1;

// 50MHz = 18.432MHz / 1152 * 3125 @ 16KHz
parameter LOOPA_50 = 1152-1;
parameter LOOPB_50 = 3125-1;

// 1MHz
parameter LOOPA_1 = 2304-1;
parameter LOOPB_1 = 125-1;

// 60MHz
parameter LOOPA_60 = 192-1;
parameter LOOPB_60 = 625-1;

// 66MHz
parameter LOOPA_66 = 1152-1;
parameter LOOPB_66 = 4125-1;

// 75MHz
parameter LOOPA_75 =  768-1;
parameter LOOPB_75 = 3125-1;

// 80MHz
parameter LOOPA_80 =  144-1;
parameter LOOPB_80 =  625-1;

// choise
parameter LOOPA = LOOPA_25;
parameter LOOPB = LOOPB_25;
parameter LOOPA_SW2 = LOOPA_33;
parameter LOOPB_SW2 = LOOPB_33;
parameter LOOPA_SW3 = LOOPA_50;
parameter LOOPB_SW3 = LOOPB_50;

wire [13:0] loopa = ~PSW[2] ? LOOPA_SW2 : ~PSW[3] ? LOOPA_SW3 : LOOPA;
wire [13:0] loopb = ~PSW[2] ? LOOPB_SW2 : ~PSW[3] ? LOOPB_SW3 : LOOPB;

wire clkd_a,clkd_b,freq_a,freq_b,phase_a,phase_b,lock;

pll pll(
  .clk_a(PCLK) ,.clk_b(vco_clk),
  .div_a(loopa),.div_b(loopb),
  .clkd_a(clkd_a),.clkd_b(clkd_b),
  .freq_a(freq_a),.freq_b(freq_b),
  .phase_a(phase_a),.phase_b(phase_b),
  .lock(lock)
);

/////////////////////////////////////////////////////////////////////////////
// PLL FEEDBACK driver
/////////////////////////////////////////////////////////////////////////////
wire n_vco_reset = PSW[1];
reg  delayed_a; // delayed phase a signal to hold time for Hi-z delay
always @(posedge vco_clk or posedge phase_a)
begin
  if(phase_a)
    delayed_a <= 1'b1;
  else
    delayed_a <= phase_a;
end

wire pll_hiz   = ~phase_a & ~phase_b & n_vco_reset;
wire pll_level = (phase_a | delayed_a) & n_vco_reset; // keep level to Hi-Z
OBUFT_F_8 feedback(.O(PLL_OUT),.I(pll_level),.T(pll_hiz));


/////////////////////////////////////////////////////////////////////////////
// Frequency Monitor
/////////////////////////////////////////////////////////////////////////////
wire monitor_clock = PSW[1] ? vco_clk : x_inv;
//wire monitor_clock = vco_clk;
wire update;
wire [23:0] freq_num;
freq_mon MINITOR(
  .base_clk(PCLK),
  .update(update),
  .target_clk(monitor_clock),
  .target_freq(freq_num)
);

// display cycle
reg disp_cycle;
always @(posedge PCLK)
begin
  if(update)
    disp_cycle <= ~disp_cycle;
end

	    
/////////////////////////////////////////////////////////////////////////////
// 7SEG-LED assign
/////////////////////////////////////////////////////////////////////////////
//
// PSW[0] : OFF = MHz / ON = KHz
//
assign led_num = PSW[0] ? freq_num[23:8] : freq_num[15:0];
assign dot[0] = 1'b0;
assign dot[1] = PSW[0] ? 1'b0 : 1'b1;
assign dot[2] = PSW[0] ? 1'b1 : 1'b0;
assign dot[3] = 1'b0;
assign led_com = ticker[16:15]; // 140Hz cycle
assign led_en  = !(ticker[14:0] == 0);

//
// static LED x8 driver
//
assign LED[0] = disp_cycle;
assign LED[1] = ~lock   ? 1'b0 : 1'bz;
assign LED[2] = freq_a  ? 1'b0 : 1'bz;
assign LED[3] = freq_b  ? 1'b0 : 1'bz;
assign LED[4] = ~PSW[3] ? 1'b0 : 1'bz;
assign LED[5] = ~PSW[2] ? 1'b0 : 1'bz;
assign LED[6] = ~PSW[1] ? 1'b0 : 1'bz;
assign LED[7] = ~PSW[0] ? 1'b0 : 1'bz;

//			   cycle
// BUZZER driver
//
assign BUZZ = 1'bz;



endmodule

